<?php

namespace Drupal\image_style_on_upload\Utility;

use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\FileInterface;
use Drupal\image\ImageStyleInterface;

/**
 * Applies the image style on file upload.
 *
 * @package Drupal\image_style_on_upload\Utility
 */
class ImageStyleApplier {

  use StringTranslationTrait;

  /**
   * The image style config.
   *
   * @var \Drupal\Core\Config\Config|\Drupal\Core\Config\ImmutableConfig|null
   */
  protected $imageStyleConfig = NULL;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private $fileSystem;

  /**
   * ImageStyleApplier constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The config factory.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system.
   */
  public function __construct(
    ConfigFactory $configFactory,
    EntityTypeManagerInterface $entityTypeManager,
    FileSystemInterface $fileSystem
  ) {
    $this->imageStyleConfig = $configFactory->get('image_style_on_upload.settings');
    $this->entityTypeManager = $entityTypeManager;
    $this->fileSystem = $fileSystem;
  }

  /**
   * Applies the file interface.
   *
   * @param \Drupal\file\FileInterface $file
   *   The filter interface.
   *
   * @return array
   *   An array of errors incase there were any.
   */
  public function apply(FileInterface $file): array {
    if (!$this->imageStyleConfig) {
      return [];
    }

    if (!$file->isTemporary()) {
      return [];
    }

    if (!in_array($file->getMimeType(), $this->getConfiguredMimeTypes())) {
      return [];
    }

    $errors = [];
    $imageStyle = $this->getConfiguredImageStyle();

    if (!$imageStyle instanceof ImageStyleInterface) {
      $errors[] = $this->t('The image style configured for the upload does not exist');
      return $errors;
    }

    $derived_uri = $file->getFileUri() . '.derived';

    if (!$imageStyle->createDerivative($file->getFileUri(), $derived_uri)) {
      $errors[] = t('The uploaded image could not be processed with style %image_style. The image file may be invalid.', ['%image_style' => $image_style_name]);
    }
    else if ($this->fileSystem->move($derived_uri, $file->getFileUri(), FileSystemInterface::EXISTS_REPLACE) === FALSE) {
      $errors[] = t('An error occurred while saving the uploaded image file.');
    }

    return $errors;
  }

  /**
   * Returns an array of the configured mime types.
   *
   * @return array
   *   An array of the configured mime types.
   */
  protected function getConfiguredMimeTypes(): array {
    $mimeTypes = $this->imageStyleConfig->get('mime_types');

    if (!is_string($mimeTypes) || empty($mimeTypes)) {
      return [];
    }

    return explode(' ', $mimeTypes);
  }

  /**
   * Returns the configured image style.
   *
   * @return \Drupal\image\ImageStyleInterface|null
   *   An image style.
   */
  private function getConfiguredImageStyle(): ?ImageStyleInterface {
    $imageStyle = $this->imageStyleConfig->get('image_style');

    if (empty($imageStyle)) {
      return NULL;
    }

    try {
      $imageStyle = $this->entityTypeManager
        ->getStorage('image_style')
        ->load($imageStyle);
    }
    catch (\Exception $e) {
      $imageStyle = NULL;
    }

    if (!$imageStyle instanceof ImageStyleInterface) {
      return NULL;
    }

    return $imageStyle;
  }

}
